# -*- coding: utf-8 -*-
#
# Copyright 2014 Thomas Amland
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

import os
import sqlite3
from mopidy.models import Track, Artist, Album, Playlist


class BansheeDB(object):

    def __init__(self, database_file, art_dir):
        self.database_file = database_file
        self.art_dir = art_dir

    def get_tracks(self):
        con = sqlite3.connect(self.database_file)
        con.row_factory = sqlite3.Row
        q = """
            SELECT CoreTracks.Title AS TrackName, CoreTracks.Uri,
            CoreTracks.TrackNumber, CoreTracks.Year, CoreTracks.Duration,
            CoreArtists.Name AS ArtistName, CoreAlbums.Title AS AlbumName,
            CoreAlbums.ArtworkID
            FROM CoreTracks
            LEFT OUTER JOIN CoreArtists ON CoreArtists.ArtistID = CoreTracks.ArtistID
            LEFT OUTER JOIN CoreAlbums ON CoreAlbums.AlbumID = CoreTracks.AlbumID
            """
        tracks = [self._create_track(row) for row in con.execute(q)]
        con.close()
        return tracks

    def get_playlists(self):
        con = sqlite3.connect(self.database_file)
        con.row_factory = sqlite3.Row
        q = """SELECT PlaylistID, Name FROM CorePlaylists"""
        playlists = [Playlist(uri='banshee:playlist:%d' % int(row[b'PlaylistID']),
                              name=row[b'Name']) for row in con.execute(q)]
        con.close()
        return playlists

    def get_playlist_tracks(self, playlist_id):
        con = sqlite3.connect(self.database_file)
        con.row_factory = sqlite3.Row
        q = """
            SELECT CoreTracks.Title AS TrackName, CoreTracks.Uri,
            CoreTracks.TrackNumber, CoreTracks.Year, CoreTracks.Duration,
            CoreArtists.Name AS ArtistName, CoreAlbums.Title AS AlbumName,
            CoreAlbums.ArtworkID
            FROM CorePlaylistEntries
            LEFT OUTER JOIN CoreTracks ON CoreTracks.TrackID = CorePlaylistEntries.TrackID
            LEFT OUTER JOIN CoreArtists ON CoreArtists.ArtistID = CoreTracks.ArtistID
            LEFT OUTER JOIN CoreAlbums ON CoreAlbums.AlbumID = CoreTracks.AlbumID
            WHERE CorePlaylistEntries.PlaylistID = ?
            """
        tracks = [self._create_track(row) for row in con.execute(q, (playlist_id,))]
        con.close()
        return tracks


    def _create_track(self, row):
        art_id = row[b'ArtworkID']
        images = [os.path.join(self.art_dir, art_id + '.jpg')] if art_id else []
        artist = Artist(name=row[b'ArtistName'],)
        album = Album(
            name=row[b'AlbumName'],
            artists=[artist],
            images=images)
        track = Track(
            name=row[b'TrackName'],
            track_no=row[b'TrackNumber'],
            artists=[artist],
            album=album,
            uri=row[b'Uri'],
            date=unicode(row[b'Year']),
            length=row[b'Duration'])
        return track


def _build_sql_query(query):
    args = []
    where = []
    if 'artist' in query:
        where.append("ArtistName = ?")
        args.append(query['artist'][0])

    if 'album' in query:
        where.append("AlbumName = ?")
        args.append(query['album'][0])

    # if 'any' in query:
    #     where.append("(ArtistName = ? OR AlbumName = ?)")
    #     args.append(query['any'])
    #     args.append(query['any'])
    q = " WHERE " + " AND ".join(where) if where else ""
    return q, args


# -*- coding: utf-8 -*-
#
# Copyright 2014 Thomas Amland
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import unicode_literals

import os
import logging
import traceback
from mopidy import config, ext
import mopidy
from mopidy.models import Playlist
from pykka import ThreadingActor
from mopidy.backend import LibraryProvider
from mopidy.local import search
import banshee

log = logging.getLogger('banshee')


class Extension(ext.Extension):
    dist_name = 'Mopidy-Banshee'
    ext_name = 'banshee'
    version = '0.1.1'

    def get_default_config(self):
        conf_file = os.path.join(os.path.dirname(__file__), 'ext.conf')
        return config.read(conf_file)

    def get_config_schema(self):
        schema = super(Extension, self).get_config_schema()
        schema['database_file'] = config.String()
        schema['art_dir'] = config.String()
        return schema

    def setup(self, registry):
        registry.add('backend', BansheeBackend)


class BansheeBackend(ThreadingActor, mopidy.backend.Backend):
    uri_schemes = ['banshee']

    def __init__(self, config, audio):
        super(BansheeBackend, self).__init__()
        self.config = config
        database_file = os.path.expanduser(config['banshee']['database_file'])
        art_dir = os.path.expanduser(config['banshee']['art_dir'])
        self.banshee_db = banshee.BansheeDB(database_file, art_dir)
        self.library = BansheeLibraryProvider(self, self.banshee_db)
        self.playlists = BansheePlaylistProvider(self, self.banshee_db)


class BansheeLibraryProvider(LibraryProvider):
    def __init__(self, backend, banshee_db):
        super(BansheeLibraryProvider, self).__init__(backend)
        self._tracks = None
        self.banshee_db = banshee_db

    def find_exact(self, query=None, uris=None):
        try:
            log.debug("banshee::find_exact(%s,%s)" % (repr(query), uris))
            if query is None:
                return None
            if self._tracks is None:
                self._tracks = self.banshee_db.get_tracks()
            return search.find_exact(self._tracks, query, uris)
        except Exception as e:
            traceback.print_exc()
            raise e

    def search(self, query=None, uris=None):
        log.debug("banshee::search(%s,%s)" % (query, uris))
        return self.find_exact(query, uris)

    def lookup(self, uri):
        log.debug("banshee::lookup(%s)" % (uri))
        raise NotImplementedError

    def refresh(self, uri=None):
        log.debug("banshee::refresh(%s)" % (uri))
        raise NotImplementedError


class BansheePlaylistProvider(mopidy.backend.PlaylistsProvider):
    def __init__(self, backend, banshee_db):
        super(BansheePlaylistProvider, self).__init__(backend)
        self.banshee_db = banshee_db
        self._playlists = banshee_db.get_playlists()

    def lookup(self, uri):
        pl_id = int(uri.split(':')[-1])
        tracks = self.banshee_db.get_playlist_tracks(pl_id)
        return Playlist(tracks=tracks)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from setuptools import setup, find_packages


setup(
    name='Mopidy-Banshee',
    version='0.1.1',
    description='Banshee extension for Mopidy',
    long_description=open('README.rst').read(),
    author='Thomas Amland',
    author_email='thomas.amland@googlemail.com',
    url='http://github.com/tamland/mopidy-banshee',
    license='GPL',
    packages=find_packages(exclude=['tests', 'tests.*']),
    zip_safe=False,
    include_package_data=True,
    test_suite='tests',
    tests_require=['pytest'],
    install_requires=['setuptools', 'Mopidy >= 0.14'],
    entry_points={
        'mopidy.ext': [
            'banshee = mopidy_banshee:Extension',
        ],
    },
    keywords='mopidy banshee',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: End Users/Desktop',
        'Environment :: No Input/Output (Daemon)',
        'Topic :: Multimedia :: Sound/Audio :: Players',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)
